@@ -1,14 +1,14 @@
 <?php
 /**
- * Copyright 2015-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2015-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @category  Horde
- * @copyright 2015-2016 Horde LLC
- * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
- * @package   Crypt
+ * @author   Michael Slusarz <slusarz@horde.org>
+ * @category Horde
+ * @license  http://www.horde.org/licenses/lgpl21 LGPL 2.1
+ * @package  Crypt
  */
 
 /**
@@ -20,10 +20,10 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2015-2016 Horde LLC
- * @internal
+ * @copyright 2015-2017 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Crypt
+ * @internal
  */
 class Horde_Crypt_Pgp_Backend_Binary
 extends Horde_Crypt_Pgp_Backend
@@ -77,6 +77,26 @@ extends Horde_Crypt_Pgp_Backend
             '--yes',
             '--homedir ' . $this->_tempdir
         );
+
+        /* GnuPG 2 requires specifying the pinentry-mode. */
+        $result = $this->_callGpg(array('--version'), 'r');
+        if (preg_match('/gpg \(GnuPG\) (\d+\.\d+\.\d+)/', $result->stdout, $version) &&
+            version_compare($version[1], '2.1.0', '>')) {
+            if (version_compare($version[1], '2.1.12', '>=')) {
+                $this->_gnupg[] = '--pinentry-mode loopback';
+                file_put_contents(
+                    $this->_tempdir . '/gpg-agent.conf',
+                    'allow-loopback-pinentry'
+                );
+            } else {
+                throw new Horde_Crypt_Exception(
+                    sprintf(
+                        Horde_Crypt_Translation::t("Unsupported GnuPG version %s detected. Only versions < 2.1 and > 2.1.11 are supported."),
+                        $version[1]
+                    )
+                );
+            }
+        }
     }
 
     /**
@@ -433,7 +453,7 @@ extends Horde_Crypt_Pgp_Backend
             $cmdline[] = $keyring;
             $cmdline[] = '--encrypt';
             foreach (array_keys($params['recips']) as $val) {
-                $cmdline[] = '--recipient ' . $val;
+                $cmdline[] = '--recipient ' . escapeshellarg($val);
             }
         } else {
             $cmdline[] = '--symmetric';
@@ -552,7 +572,7 @@ extends Horde_Crypt_Pgp_Backend
             '--armor',
             '--always-trust',
             '--batch',
-            '--charset ' . (isset($params['charset']) ? $params['charset'] : 'UTF-8'),
+            '--charset ' . (isset($params['charset']) ? escapeshellarg($params['charset']) : 'UTF-8'),
             $keyring,
             '--verify'
         );
