@@ -1,8 +1,6 @@
 <?php
 /**
- * Library to interact with the OpenSSL library and implement S/MIME.
- *
- * Copyright 2002-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -13,6 +11,17 @@
  * @license  http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package  Crypt
  */
+
+/**
+ * Library to interact with the OpenSSL library and implement S/MIME.
+ *
+ * @author    Mike Cochrane <mike@graftonhall.co.nz>
+ * @author    Michael Slusarz <slusarz@horde.org>
+ * @category  Horde
+ * @copyright 2002-2017 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
+ * @package   Crypt
+ */
 class Horde_Crypt_Smime extends Horde_Crypt
 {
     /**
@@ -235,8 +244,9 @@ class Horde_Crypt_Smime extends Horde_Crypt
             'protocol',
             'application/pkcs7-signature'
         );
-        // Per RFC 5751 [3.4.3.2], 'sha1' has been deprecated for 'sha-1'.
-        $smime_part->setContentTypeParameter('micalg', 'sha-1');
+        $smime_part->setContentTypeParameter(
+            'micalg', $mime_message->getContentTypeParameter('micalg')
+        );
         $smime_part->addPart($mime_part);
         $smime_part->addPart($smime_sign);
 
